﻿using System;
using MVR.FileManagementSecure;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;
using SimpleJSON;

namespace ky1001
{
    public class PresetLoader : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "ky1001";
        public string pluginName = "PresetLoader";
        public string pluginVersion = "9.0";
        public string pluginDate = "2022-09-28";
        public string pluginDescription = "";
        #endregion

        private Atom _targetAtom;

        private const string _clothingPresetPath = "Custom/Atom/Person/Clothing";
        private const string _hairPresetPath = "Custom/Atom/Person/Hair";
        private const string _posePresetPath = "Custom/Atom/Person/Pose";
        private const string _skinPresetPath = "Custom/Atom/Person/Skin";
        private const string _appearancePresetPath = "Custom/Atom/Person/Appearance";
        private const string _addonPackagesPath = "AddonPackages";
        private const string _filePrefs = "AddonPackagesFilePrefs/";
        private const string _pluginDataPath = "Saves/PluginData/ky1001/PresetLoader";
        private const string _clothing = "Clothing";
        private const string _hair = "Hair";
        private const string _pose = "Pose";
        private const string _skin = "Skin";
        private const string _appearance = "Appearance";
        private const int _jsonstringLimit = 15000;
        private const float _currentPathHeight = 10f;
        private const float _presetPathHeight = 165f;

        private JSONStorableBool _enableRnd;
        private JSONStorableBool _enableAutoLoad;
        private JSONStorableBool _enableSubfolder;
        private JSONStorableBool _enablePositionAdjust;
        private Dictionary<string, JSONStorableBool> _enablePresetDic = new Dictionary<string, JSONStorableBool>();
        private Dictionary<string, JSONStorableBool> _onlyFavDic = new Dictionary<string, JSONStorableBool>();
        private Dictionary<string, JSONStorableString> _savedPathDic = new Dictionary<string, JSONStorableString>();
        private Dictionary<string, UIDynamicButton> _setPresetPathButton = new Dictionary<string, UIDynamicButton>();
        private Dictionary<string, UIDynamicToggle> _enablePresetToggle = new Dictionary<string, UIDynamicToggle>();
        private Dictionary<string, UIDynamicToggle> _onlyFavToggle = new Dictionary<string, UIDynamicToggle>();
        private Dictionary<string, UIDynamicTextField> _currentText = new Dictionary<string, UIDynamicTextField>();
        private Dictionary<string, UIDynamicTextField> _pathText = new Dictionary<string, UIDynamicTextField>();
        private Dictionary<string, string> _presetSystemPath = new Dictionary<string, string>();
        private Dictionary<string, JSONStorableString> _filesInFolder = new Dictionary<string, JSONStorableString>();
        private Dictionary<string, JSONStorableString> _current = new Dictionary<string, JSONStorableString>();
        private Dictionary<string, string> _presetPathDic = new Dictionary<string, string>();
        private JSONStorableStringChooser _selectCount;
        private UIDynamicButton _loadSettingButton;
        private UIDynamicButton _saveSettingButton;
        private UIDynamic _spLeft;
        private UIDynamic _spRight;
        private Dictionary<string, string> _favPathDic = new Dictionary<string, string>();
        private Dictionary<string, List<string>> _listDic = new Dictionary<string, List<string>>();
        private Dictionary<string, List<string>> _presetListDic = new Dictionary<string, List<string>>();
        private Dictionary<string, List<string>> _favListDic = new Dictionary<string, List<string>>();
        private List<string> _presetList = new List<string>();
        private List<string> _favList = new List<string>();
        private List<string> _femaleList = new List<string>();
        private List<string> _activeClothingList = new List<string>();
        private string _presetName;
        private string _presetPath;
        private string _varFileName;
        private string _varFullName;
        private string _varFullPath;
        private Dictionary<string, int> _previousNumber = new Dictionary<string, int>();
        private string _previousSelectCount;
        private bool _isTopMenu = true;
        private bool _loopStart = false;
        private float _waitCount;
        private bool _isFemale;
        private string _posePresetFile;
        private string _skinPresetFile;
        private string _loadData;
        private JSONNode _jSONNode;
        private JSONArray _asArray;
        private JSONClass _asObject;
        private int _startTime;
        private int _currentTime;
        private int _selectTime;
        private string _id;
        private Vector3 _personPosi;
        private Vector3 _loadedPosi;
        private bool _enablePush;
        private bool _isPosition;
        private bool _isHip;
        private float _min;
        private Vector3 _control;
        private Vector3 _hipControl;
        private Vector3 _posi;
        private Vector3 _rota;
        private Dictionary<string, Vector3> _rigidbodyPosiLocal = new Dictionary<string, Vector3>();
        private Dictionary<string, Quaternion> _rigidbodyRotaLocal = new Dictionary<string, Quaternion>();
        private Dictionary<string, Vector3> _rigidbodyPosiWorld = new Dictionary<string, Vector3>();
        private Dictionary<string, Quaternion> _rigidbodyRotaWorld = new Dictionary<string, Quaternion>();
        private Vector3 _rootPosi;
        private Vector3 _rootRota;

        private readonly string[] _female = new string[]
        {
            "Candy",
            "Evey",
            "Janie",
            "Kayla",
            "Lexi",
            "Maria",
            "Mia",
            "Simone",
            "Tara",
            "Tina"
        };

        private readonly string[] _rigidbodyNames = new string[]
        {
            "hip",
            "pelvis",

            "rThigh",
            "rShin",
            "rFoot",
            "rToe",

            "lThigh",
            "lShin",
            "lFoot",
            "lToe",

            "abdomen",
            "abdomen2",

            "chest",

            "lPectoral",
            "rPectoral",

            "rCollar",
            "rShldr",
            "rForeArm",
            "rHand",

            "lCollar",
            "lShldr",
            "lForeArm",
            "lHand",

            "neck",
            "head"
        };

        private readonly string[] _presets = new string[]
        {
            "Clothing",
            "Hair",
            "Pose",
            "Skin",
            "Appearance"
        };

        public override void Init()
        {
            try
            {
                _targetAtom = containingAtom;

                if (_targetAtom.type != "Person")
                {
                    SuperController.LogError("Please attach this plugin to Person");
                    return;
                }

                var loadPresetButton = CreateButton("Load Presets");
                loadPresetButton.button.onClick.AddListener(LoadPreset);
                JSONStorableAction LoadPresetAct = new JSONStorableAction("Load Presets", LoadPreset);
                _enablePush = true;

                var changeManuButton = CreateButton("Change Menu", true);
                changeManuButton.button.onClick.AddListener(ChangeMenu);
                changeManuButton.buttonColor = Color.yellow;
                JSONStorableAction ChangeMenuAct = new JSONStorableAction("Change Menu", ChangeMenu);
                _isTopMenu = true;

                _enableAutoLoad = new JSONStorableBool("Enable Auto Load", false);
                CreateToggle(_enableAutoLoad, true);

                _enableRnd = new JSONStorableBool("Enable Random Load", true);
                var enableRnd = CreateToggle(_enableRnd);
                enableRnd.toggle.onValueChanged.AddListener(delegate { ResetCount(); });

                _enablePositionAdjust = new JSONStorableBool("Enable Position Adjustment", true);
                CreateToggle(_enablePositionAdjust, false);

                _enableSubfolder = new JSONStorableBool("Include Subfolder", true);
                var enableSubfolderToggle = CreateToggle(_enableSubfolder);
                enableSubfolderToggle.toggle.onValueChanged.AddListener(delegate { RefreshList(); });

                List<string> waitCount = new List<string>();
                waitCount.Add("5");
                waitCount.Add("15");
                waitCount.Add("30");
                waitCount.Add("60");
                waitCount.Add("120");
                waitCount.Add("180");

                _selectCount = new JSONStorableStringChooser("Interval", waitCount, "30", "Interval [sec]");
                var sc = CreatePopup(_selectCount, true);

                var sp1 = CreateSpacer(true);
                sp1.height = 21f;

                foreach (var item in _presets)
                {
                    _favPathDic[item] = "";
                    _favListDic[item] = new List<string>();
                }

                var presetName = _clothing;
                _enablePresetDic[presetName] = new JSONStorableBool("Enable " + presetName + " Presets", false);
                _onlyFavDic[presetName] = new JSONStorableBool("Favorite " + presetName + " Only", false);
                _current[presetName] = new JSONStorableString("", "");
                _filesInFolder[presetName] = new JSONStorableString("", "");
                MakeUi(presetName, SetClothingPath);
                JSONStorableAction SetClothingPresetPathAct = new JSONStorableAction("Set " + presetName + " Presets Path", SetClothingPathAct);

                presetName = _hair;
                _enablePresetDic[presetName] = new JSONStorableBool("Enable " + presetName + " Presets", false);
                _onlyFavDic[presetName] = new JSONStorableBool("Favorite " + presetName + " Only", false);
                _current[presetName] = new JSONStorableString("", "");
                _filesInFolder[presetName] = new JSONStorableString("", "");
                MakeUi(presetName, SetHairPath);
                JSONStorableAction SetHairPresetPathAct = new JSONStorableAction("Set " + presetName + " Presets Path", SetHairPathAct);

                presetName = _pose;
                _enablePresetDic[presetName] = new JSONStorableBool("Enable " + presetName + " Presets", false);
                _onlyFavDic[presetName] = new JSONStorableBool("Favorite " + presetName + " Only", false);
                _current[presetName] = new JSONStorableString("", "");
                _filesInFolder[presetName] = new JSONStorableString("", "");
                MakeUi(presetName, SetPosePath);
                JSONStorableAction SetPosePresetPathAct = new JSONStorableAction("Set " + presetName + " Presets Path", SetPosePathAct);

                presetName = _skin;
                _enablePresetDic[presetName] = new JSONStorableBool("Enable " + presetName + " Presets", false);
                _onlyFavDic[presetName] = new JSONStorableBool("Favorite " + presetName + " Only", false);
                _current[presetName] = new JSONStorableString("", "");
                _filesInFolder[presetName] = new JSONStorableString("", "");
                MakeUi(presetName, SetSkinPath);
                JSONStorableAction SetSkinPresetPathAct = new JSONStorableAction("Set " + presetName + " Presets Path", SetSkinPathAct);

                presetName = _appearance;
                _enablePresetDic[presetName] = new JSONStorableBool("Enable " + presetName + " Presets", false);
                _onlyFavDic[presetName] = new JSONStorableBool("Favorite " + presetName + " Only", false);
                _current[presetName] = new JSONStorableString("", "");
                _filesInFolder[presetName] = new JSONStorableString("", "");
                MakeUi(presetName, SetAppearancePath);
                JSONStorableAction SetAppearancePresetPathAct = new JSONStorableAction("Set " + presetName + " Presets Path", SetAppearancePathAct);
                RemoveItems(_appearance);

                _presetSystemPath[_clothing] = _clothingPresetPath;
                _presetSystemPath[_hair] = _hairPresetPath;
                _presetSystemPath[_pose] = _posePresetPath;
                _presetSystemPath[_skin] = _skinPresetPath;
                _presetSystemPath[_appearance] = _appearancePresetPath;

                _savedPathDic[_clothing] = new JSONStorableString("Clothing Presets Path", "");
                _savedPathDic[_hair] = new JSONStorableString("Hair Presets Path", "");
                _savedPathDic[_pose] = new JSONStorableString("Pose Preset Path", "");
                _savedPathDic[_skin] = new JSONStorableString("Skin Preset Path", "");
                _savedPathDic[_appearance] = new JSONStorableString("Appearance Preset Path", "");

                _femaleList = new List<string>(_female);
                _selectTime = int.Parse(_selectCount.val);

                RegisterAction(LoadPresetAct);
                RegisterBool(_enableRnd);
                RegisterBool(_enablePositionAdjust);
                RegisterBool(_enableSubfolder);
                RegisterBool(_enableAutoLoad);
                RegisterStringChooser(_selectCount);
                RegisterAction(SetClothingPresetPathAct);
                RegisterAction(SetHairPresetPathAct);
                RegisterAction(SetPosePresetPathAct);
                RegisterAction(SetSkinPresetPathAct);
                RegisterAction(SetAppearancePresetPathAct);
                RegisterBool(_enablePresetDic[_clothing]);
                RegisterBool(_enablePresetDic[_hair]);
                RegisterBool(_enablePresetDic[_pose]);
                RegisterBool(_enablePresetDic[_skin]);
                RegisterBool(_enablePresetDic[_appearance]);
                RegisterBool(_onlyFavDic[_clothing]);
                RegisterBool(_onlyFavDic[_hair]);
                RegisterBool(_onlyFavDic[_pose]);
                RegisterBool(_onlyFavDic[_skin]);
                RegisterBool(_onlyFavDic[_appearance]);
                RegisterString(_savedPathDic[_clothing]);
                RegisterString(_savedPathDic[_hair]);
                RegisterString(_savedPathDic[_pose]);
                RegisterString(_savedPathDic[_skin]);
                RegisterString(_savedPathDic[_appearance]);
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        private void ChangeMenu()
        {
            foreach (var item in _presets)
            {
                RemoveItems(item);
            }

            if (_loadSettingButton != null)
            {
                RemoveButton(_loadSettingButton);
            }

            if (_saveSettingButton != null)
            {
                RemoveButton(_saveSettingButton);
            }

            if (_spLeft != null)
            {
                RemoveSpacer(_spLeft);
            }

            if (_spRight != null)
            {
                RemoveSpacer(_spRight);
            }

            if (_isTopMenu)
            {
                _loadSettingButton = CreateButton("Load Setting", false);
                _loadSettingButton.buttonColor = Color.cyan;
                _saveSettingButton = CreateButton("Save Setting", true);
                _saveSettingButton.buttonColor = Color.cyan;
                _spLeft = CreateSpacer();
                _spLeft.height = 10;
                _spRight = CreateSpacer(true);
                _spRight.height = 10;

                _isTopMenu = false;
                _presetName = _appearance;
                MakeUi(_presetName, SetAppearancePath);

                _loadSettingButton.button.onClick.AddListener(() =>
                {
                    if (!FileManagerSecure.DirectoryExists(_pluginDataPath))
                    {
                        FileManagerSecure.CreateDirectory(_pluginDataPath);
                    }
                    var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_pluginDataPath, false, false, true, true);
                    SuperController.singleton.GetMediaPathDialog(LoadSetting, "json", _pluginDataPath, true, true, false, null, false, shortcuts);
                });

                _saveSettingButton.button.onClick.AddListener(() =>
                {
                    if (!FileManagerSecure.DirectoryExists(_pluginDataPath))
                    {
                        FileManagerSecure.CreateDirectory(_pluginDataPath);
                    }
                    var fileBrowserUI = SuperController.singleton.fileBrowserUI;
                    fileBrowserUI.SetTitle("Save PresetLoader Setting");
                    fileBrowserUI.fileRemovePrefix = null;
                    fileBrowserUI.hideExtension = false;
                    fileBrowserUI.keepOpen = false;
                    fileBrowserUI.fileFormat = "json";
                    fileBrowserUI.defaultPath = _pluginDataPath;
                    fileBrowserUI.showDirs = true;
                    fileBrowserUI.shortCuts = null;
                    fileBrowserUI.browseVarFilesAsDirectories = true;
                    fileBrowserUI.SetTextEntry(true);
                    fileBrowserUI.Show(SaveSetting);
                    fileBrowserUI.fileEntryField.text = (int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds + ".json";
                    fileBrowserUI.ActivateFileNameField();
                    fileBrowserUI.showInstallFolderInDirectoryList = true;
                });
            }
            else
            {
                _isTopMenu = true;
                _presetName = _clothing;
                MakeUi(_presetName, SetClothingPath);

                _presetName = _hair;
                MakeUi(_presetName, SetHairPath);

                _presetName = _pose;
                MakeUi(_presetName, SetPosePath);

                _presetName = _skin;
                MakeUi(_presetName, SetSkinPath);
            }
        }

        private void RemoveItems(string item)
        {
            if (_setPresetPathButton[item] != null)
            {
                RemoveButton(_setPresetPathButton[item]);
            }

            if (_enablePresetToggle[item] != null)
            {
                RemoveToggle(_enablePresetToggle[item]);
            }

            if (_onlyFavToggle[item] != null)
            {
                RemoveToggle(_onlyFavToggle[item]);
            }

            if (_currentText[item] != null)
            {
                RemoveTextField(_currentText[item]);
            }

            if (_pathText[item] != null)
            {
                RemoveTextField(_pathText[item]);
            }
        }

        public void MakeUi(string presetName, UnityAction action)
        {
            _setPresetPathButton[presetName] = CreateButton("Set " + presetName + " Presets Path");
            _setPresetPathButton[presetName].button.onClick.AddListener(action);

            _enablePresetToggle[presetName] = CreateToggle(_enablePresetDic[presetName]);
            _onlyFavToggle[presetName] = CreateToggle(_onlyFavDic[presetName], true);
            _onlyFavToggle[presetName].toggle.onValueChanged.AddListener(delegate { ChangePath(presetName); });

            _currentText[presetName] = CreateTextField(_current[presetName]);
            _currentText[presetName].height = _currentPathHeight;
            _pathText[presetName] = CreateTextField(_filesInFolder[presetName], true);
            _pathText[presetName].height = _presetPathHeight;
        }

        public void LoadSetting(string path)
        {
            if (path != null && path != "")
            {
                var loadData = SuperController.singleton.ReadFileIntoString(path);
                JSONNode jSONNode = JSON.Parse(loadData);
                _enableRnd.val = bool.Parse(jSONNode["Enable Random Load"]);
                _enablePositionAdjust.val = bool.Parse(jSONNode["Enable Position Adjustment"]);
                _enableSubfolder.val = bool.Parse(jSONNode["Include Subfolder"]);
                _enableAutoLoad.val = bool.Parse(jSONNode["Enable Auto Load"]);
                _selectCount.val = jSONNode["Interval"];

                _presetName = _clothing;
                _enablePresetDic[_presetName].val = bool.Parse(jSONNode[_presetName][0]);
                _onlyFavDic[_presetName].val = bool.Parse(jSONNode[_presetName][1]);
                _savedPathDic[_presetName].val = jSONNode[_presetName][2];
                _presetPath = _presetSystemPath[_presetName];
                var tempPath = MakeFilename(_savedPathDic[_presetName].val);
                LoadPresetsList(tempPath);
                if (_savedPathDic[_presetName].val == "")
                {
                    _favPathDic[_presetName] = "";
                    _favListDic[_presetName] = new List<string>();
                    _presetPathDic[_presetName] = "";
                    _presetListDic[_presetName] = new List<string>();
                    _enablePresetDic[_presetName].val = false;
                }
                ChangePath(_presetName);

                _presetName = _hair;
                _enablePresetDic[_presetName].val = bool.Parse(jSONNode[_presetName][0]);
                _onlyFavDic[_presetName].val = bool.Parse(jSONNode[_presetName][1]);
                _savedPathDic[_presetName].val = jSONNode[_presetName][2];
                _presetPath = _presetSystemPath[_presetName];
                tempPath = MakeFilename(_savedPathDic[_presetName].val);
                LoadPresetsList(tempPath);
                if (_savedPathDic[_presetName].val == "")
                {
                    _favPathDic[_presetName] = "";
                    _favListDic[_presetName] = new List<string>();
                    _presetPathDic[_presetName] = "";
                    _presetListDic[_presetName] = new List<string>();
                    _enablePresetDic[_presetName].val = false;
                }
                ChangePath(_presetName);

                _presetName = _pose;
                _enablePresetDic[_presetName].val = bool.Parse(jSONNode[_presetName][0]);
                _onlyFavDic[_presetName].val = bool.Parse(jSONNode[_presetName][1]);
                _savedPathDic[_presetName].val = jSONNode[_presetName][2];
                _presetPath = _presetSystemPath[_presetName];
                tempPath = MakeFilename(_savedPathDic[_presetName].val);
                LoadPresetsList(tempPath);
                if (_savedPathDic[_presetName].val == "")
                {
                    _favPathDic[_presetName] = "";
                    _favListDic[_presetName] = new List<string>();
                    _presetPathDic[_presetName] = "";
                    _presetListDic[_presetName] = new List<string>();
                    _enablePresetDic[_presetName].val = false;
                }
                ChangePath(_presetName);

                _presetName = _skin;
                _enablePresetDic[_presetName].val = bool.Parse(jSONNode[_presetName][0]);
                _onlyFavDic[_presetName].val = bool.Parse(jSONNode[_presetName][1]);
                _savedPathDic[_presetName].val = jSONNode[_presetName][2];
                _presetPath = _presetSystemPath[_presetName];
                tempPath = MakeFilename(_savedPathDic[_presetName].val);
                LoadPresetsList(tempPath);
                if (_savedPathDic[_presetName].val == "")
                {
                    _favPathDic[_presetName] = "";
                    _favListDic[_presetName] = new List<string>();
                    _presetPathDic[_presetName] = "";
                    _presetListDic[_presetName] = new List<string>();
                    _enablePresetDic[_presetName].val = false;
                }
                ChangePath(_presetName);

                _presetName = _appearance;
                _enablePresetDic[_presetName].val = bool.Parse(jSONNode[_presetName][0]);
                _onlyFavDic[_presetName].val = bool.Parse(jSONNode[_presetName][1]);
                _savedPathDic[_presetName].val = jSONNode[_presetName][2];
                _presetPath = _presetSystemPath[_presetName];
                tempPath = MakeFilename(_savedPathDic[_presetName].val);
                LoadPresetsList(tempPath);
                if (_savedPathDic[_presetName].val == "")
                {
                    _favPathDic[_presetName] = "";
                    _favListDic[_presetName] = new List<string>();
                    _presetPathDic[_presetName] = "";
                    _presetListDic[_presetName] = new List<string>();
                    _enablePresetDic[_presetName].val = false;
                }
                ChangePath(_presetName);
            }
        }

        public void SaveSetting(string path)
        {
            if (path != null && path != "")
            {
                JSONClass saveData = new JSONClass();
                saveData["Enable Random Load"] = _enableRnd.val.ToString();
                saveData["Enable Position Adjustment"] = _enablePositionAdjust.val.ToString();
                saveData["Include Subfolder"] = _enableSubfolder.val.ToString();
                saveData["Enable Auto Load"] = _enableAutoLoad.val.ToString();
                saveData["Interval"] = _selectCount.val;

                _presetName = _clothing;
                saveData[_presetName]["Enable Preset"] = _enablePresetDic[_presetName].val.ToString();
                saveData[_presetName]["Favorite Only"] = _onlyFavDic[_presetName].val.ToString();
                saveData[_presetName]["Preset Path"] = _savedPathDic[_presetName].val.ToString();

                _presetName = _hair;
                saveData[_presetName]["Enable Preset"] = _enablePresetDic[_presetName].val.ToString();
                saveData[_presetName]["Favorite Only"] = _onlyFavDic[_presetName].val.ToString();
                saveData[_presetName]["Preset Path"] = _savedPathDic[_presetName].val.ToString();

                _presetName = _pose;
                saveData[_presetName]["Enable Preset"] = _enablePresetDic[_presetName].val.ToString();
                saveData[_presetName]["Favorite Only"] = _onlyFavDic[_presetName].val.ToString();
                saveData[_presetName]["Preset Path"] = _savedPathDic[_presetName].val.ToString();

                _presetName = _skin;
                saveData[_presetName]["Enable Preset"] = _enablePresetDic[_presetName].val.ToString();
                saveData[_presetName]["Favorite Only"] = _onlyFavDic[_presetName].val.ToString();
                saveData[_presetName]["Preset Path"] = _savedPathDic[_presetName].val.ToString();

                _presetName = _appearance;
                saveData[_presetName]["Enable Preset"] = _enablePresetDic[_presetName].val.ToString();
                saveData[_presetName]["Favorite Only"] = _onlyFavDic[_presetName].val.ToString();
                saveData[_presetName]["Preset Path"] = _savedPathDic[_presetName].val.ToString();

                if (path.Length < 5 || path.Substring(path.Length - 5) != ".json")
                {
                    path = path + ".json";
                }

                SaveJSON(saveData, path);
            }
        }

        public void ChangePath(string presetName)
        {
            if (presetName == _clothing)
            {
                if (_onlyFavDic[presetName].val)
                {
                    _filesInFolder[presetName].val = _favPathDic[presetName];
                    _listDic[presetName] = new List<string>(_favListDic[presetName]);
                }
                else
                {
                    _filesInFolder[presetName].val = _presetPathDic[presetName];
                    _listDic[presetName] = new List<string>(_presetListDic[presetName]);
                }
                _current[presetName].val = "";
                _previousNumber[presetName] = 0;
            }
            else if (presetName == _hair)
            {
                if (_onlyFavDic[presetName].val)
                {
                    _filesInFolder[presetName].val = _favPathDic[presetName];
                    _listDic[presetName] = new List<string>(_favListDic[presetName]);
                }
                else
                {
                    _filesInFolder[presetName].val = _presetPathDic[presetName];
                    _listDic[presetName] = new List<string>(_presetListDic[presetName]);
                }
                _current[presetName].val = "";
                _previousNumber[presetName] = 0;
            }
            else if (presetName == _pose)
            {
                if (_onlyFavDic[presetName].val)
                {
                    _filesInFolder[presetName].val = _favPathDic[presetName];
                    _listDic[presetName] = new List<string>(_favListDic[presetName]);
                }
                else
                {
                    _filesInFolder[presetName].val = _presetPathDic[presetName];
                    _listDic[presetName] = new List<string>(_presetListDic[presetName]);
                }
                _current[presetName].val = "";
                _previousNumber[presetName] = 0;
            }
            else if (presetName == _skin)
            {
                if (_onlyFavDic[presetName].val)
                {
                    _filesInFolder[presetName].val = _favPathDic[presetName];
                    _listDic[presetName] = new List<string>(_favListDic[presetName]);
                }
                else
                {
                    _filesInFolder[presetName].val = _presetPathDic[presetName];
                    _listDic[presetName] = new List<string>(_presetListDic[presetName]);
                }
                _current[presetName].val = "";
                _previousNumber[presetName] = 0;
            }
            else if (presetName == _appearance)
            {
                if (_onlyFavDic[presetName].val)
                {
                    _filesInFolder[presetName].val = _favPathDic[presetName];
                    _listDic[presetName] = new List<string>(_favListDic[presetName]);
                }
                else
                {
                    _filesInFolder[presetName].val = _presetPathDic[presetName];
                    _listDic[presetName] = new List<string>(_presetListDic[presetName]);
                }
                _current[presetName].val = "";
                _previousNumber[presetName] = 0;
            }
        }

        public void ResetCount()
        {
            foreach (var presetName in _presets)
            {
                _previousNumber[presetName] = 0;
            }
        }

        public void LoadPreset()
        {
            /*
            if (!_enablePush)
            {
                return;
            }
            else
            {
                _enablePush = false;
            }
            */

            int number;
            if (_enableAutoLoad.val)
            {
                if (_loopStart)
                {
                    _waitCount = _selectTime;
                }
                else
                {
                    _loopStart = true;
                    return;
                }
            }
            else
            {
                _loopStart = false;
                _waitCount = 0;
            }

            var presetName = _appearance;
            if (_enablePresetDic[presetName].val)
            {
                if (_listDic[presetName].Count != 0)
                {
                    if (_listDic[presetName].Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_enableRnd.val)
                        {
                            do
                            {
                                number = UnityEngine.Random.Range(0, _listDic[presetName].Count);
                            }
                            while (_previousNumber[presetName] == number);
                            _previousNumber[presetName] = number;
                        }
                        else
                        {
                            number = _previousNumber[presetName];
                            _previousNumber[presetName] = number + 1;
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _previousNumber[presetName] = 0;
                            }
                        }
                    }

                    _targetAtom.GetStorableByID("AppearancePresets").GetStringJSONParam("presetName").val = _listDic[presetName][number];
                    _targetAtom.GetStorableByID("AppearancePresets").CallAction("LoadPreset");
                    _current[presetName].val = _listDic[presetName][number];
                }
            }

            presetName = _hair;
            if (_enablePresetDic[presetName].val)
            {
                if (_listDic[presetName].Count != 0)
                {
                    if (_listDic[presetName].Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_enableRnd.val)
                        {
                            do
                            {
                                number = UnityEngine.Random.Range(0, _listDic[presetName].Count);
                            }
                            while (_previousNumber[presetName] == number);
                            _previousNumber[presetName] = number;
                        }
                        else
                        {
                            number = _previousNumber[presetName];
                            _previousNumber[presetName] = number + 1;
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _previousNumber[presetName] = 0;
                            }
                        }
                    }

                    _targetAtom.GetStorableByID("HairPresets").GetStringJSONParam("presetName").val = _listDic[presetName][number];
                    _targetAtom.GetStorableByID("HairPresets").CallAction("LoadPreset");
                    _current[presetName].val = _listDic[presetName][number];
                }
            }

            presetName = _pose;
            if (_enablePresetDic[presetName].val)
            {
                if (_listDic[presetName].Count != 0)
                {
                    if (_listDic[presetName].Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_enableRnd.val)
                        {
                            do
                            {
                                number = UnityEngine.Random.Range(0, _listDic[presetName].Count);
                            }
                            while (_previousNumber[presetName] == number);
                            _previousNumber[presetName] = number;
                        }
                        else
                        {
                            number = _previousNumber[presetName];
                            _previousNumber[presetName] = number + 1;
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _previousNumber[presetName] = 0;
                            }
                        }
                    }

                    if (_listDic[presetName][number].Contains(":"))
                    {
                        var posePresetSplit = _listDic[presetName][number].Split(':');
                        _posePresetFile = posePresetSplit[0] + ":/" + _presetSystemPath[presetName] + "/" + posePresetSplit[1] + ".vap";
                    }
                    else
                    {
                        _posePresetFile = _presetSystemPath[presetName] + "/" + _listDic[presetName][number] + ".vap";
                    }

                    int iPosi = _posePresetFile.LastIndexOf("/");
                    _posePresetFile = _posePresetFile.Insert(iPosi + 1, "Preset_");
                    _loadData = SuperController.singleton.ReadFileIntoString(_posePresetFile);
                    _jSONNode = JSON.Parse(_loadData);
                    _asArray = _jSONNode["storables"].AsArray;

                    _isPosition = false;
                    _isHip = false;
                    _control = Vector3.zero;

                    _rigidbodyPosiLocal = new Dictionary<string, Vector3>();
                    _rigidbodyRotaLocal = new Dictionary<string, Quaternion>();
                    _rigidbodyPosiWorld = new Dictionary<string, Vector3>();
                    _rigidbodyRotaWorld = new Dictionary<string, Quaternion>();

                    for (int i = 0; i < _asArray.Count; i++)
                    {
                        _asObject = _asArray[i].AsObject;
                        _id = _asObject["id"];
                        if (_id == "control" && _asObject["position"] != null)
                        {
                            _isPosition = true;
                            _posi.x = float.Parse(_asObject["position"][0]);
                            _posi.y = float.Parse(_asObject["position"][1]);
                            _posi.z = float.Parse(_asObject["position"][2]);
                            _control = _posi;
                        }
                        else if (_id == "hipControl" && _asObject["localPosition"] != null)
                        {
                            _posi.x = float.Parse(_asObject["localPosition"][0]);
                            _posi.y = float.Parse(_asObject["localPosition"][1]);
                            _posi.z = float.Parse(_asObject["localPosition"][2]);
                            _hipControl = _posi;
                        }
                        else if (_id == "hip" && _asObject["rootPosition"] != null)
                        {
                            _posi.x = float.Parse(_asObject["rootPosition"][0]);
                            _posi.y = float.Parse(_asObject["rootPosition"][1]);
                            _posi.z = float.Parse(_asObject["rootPosition"][2]);
                            _rota.x = float.Parse(_asObject["rootRotation"][0]);
                            _rota.y = float.Parse(_asObject["rootRotation"][1]);
                            _rota.z = float.Parse(_asObject["rootRotation"][2]);
                            _rigidbodyPosiLocal[_id] = _posi;
                            _rigidbodyRotaLocal[_id] = Quaternion.Euler(_rota);
                            _rigidbodyPosiWorld[_id] = _posi;
                            _rigidbodyRotaWorld[_id] = Quaternion.Euler(_rota);
                            _rootPosi = _posi;
                            _rootRota = _rota;
                            _isHip = true;
                        }
                        else if (Array.IndexOf(_rigidbodyNames, _id) >= 0 && _asObject["position"] != null)
                        {
                            _posi.x = float.Parse(_asObject["position"][0]);
                            _posi.y = float.Parse(_asObject["position"][1]);
                            _posi.z = float.Parse(_asObject["position"][2]);
                            _rota.x = float.Parse(_asObject["rotation"][0]);
                            _rota.y = float.Parse(_asObject["rotation"][1]);
                            _rota.z = float.Parse(_asObject["rotation"][2]);
                            _rigidbodyPosiLocal[_id] = _posi;
                            _rigidbodyRotaLocal[_id] = Quaternion.Euler(_rota);

                            GameObject parent = _targetAtom.GetStorableByID(_id).transform.parent.gameObject;

                            GameObject parentRigid = new GameObject();
                            GameObject currentRigid = new GameObject();

                            parentRigid.transform.position = _rigidbodyPosiWorld[parent.name];
                            parentRigid.transform.rotation = _rigidbodyRotaWorld[parent.name];
                            currentRigid.transform.position = parentRigid.transform.TransformPoint(_posi);
                            currentRigid.transform.rotation = _rigidbodyRotaWorld[parent.name] * _rigidbodyRotaLocal[_id];
                            _rigidbodyPosiWorld[_id] = currentRigid.transform.position;
                            _rigidbodyRotaWorld[_id] = currentRigid.transform.rotation;
                        }
                    }

                    if (_isHip)
                    {
                        float minY = 2f;
                        string minName = "hip";
                        foreach (var item in _rigidbodyNames)
                        {
                            if (_rigidbodyPosiWorld[item] != null)
                            {
                                float thisY = _rigidbodyPosiWorld[item].y;
                                if (minY > thisY)
                                {
                                    minY = thisY;
                                    minName = item;
                                }
                            }
                        }
                        _min = -_rootPosi.y + minY + _hipControl.y;
                    }
                    else
                    {
                        _min = 1f;
                    }

                    _personPosi = _targetAtom.GetStorableByID("control").transform.position;
                    if ((_min < 0 && _enablePositionAdjust.val) || _isPosition)
                    {
                        _activeClothingList = new List<string>();
                        _targetAtom.tempDisableRender = !_targetAtom.tempDisableRender;
                        var allGeometry = _targetAtom.GetStorableByID("geometry").GetBoolParamNames();
                        foreach (var item in allGeometry)
                        {
                            if (item.Contains("clothing:"))
                            {
                                if (_targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val)
                                {
                                    _targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val = false;
                                    _activeClothingList.Add(item);
                                }
                            }
                        }
                    }

                    _targetAtom.GetStorableByID("PosePresets").GetStringJSONParam("presetName").val = _listDic[presetName][number];
                    _targetAtom.GetStorableByID("PosePresets").CallAction("LoadPreset");
                    _loadedPosi = _targetAtom.GetStorableByID("control").transform.position;
                    _current[presetName].val = _listDic[presetName][number];

                    if ((_min < 0 && _enablePositionAdjust.val) || _isPosition)
                    {
                        StartCoroutine(MoveRoot());
                    }
                }
            }

            presetName = _skin;
            if (_enablePresetDic[presetName].val)
            {
                if (_listDic[presetName].Count != 0)
                {
                    if (_listDic[presetName].Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_enableRnd.val)
                        {
                            do
                            {
                                number = UnityEngine.Random.Range(0, _listDic[presetName].Count);
                            }
                            while (_previousNumber[presetName] == number);
                            _previousNumber[presetName] = number;
                        }
                        else
                        {
                            number = _previousNumber[presetName];
                            _previousNumber[presetName] = number + 1;
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _previousNumber[presetName] = 0;
                            }
                        }
                    }

                    var characterName = _targetAtom.GetStorableByID("geometry").GetStringChooserJSONParam("characterSelection").val;
                    if (_femaleList.Contains(characterName) || characterName.Contains("Female"))
                    {
                        _isFemale = true;
                    }
                    else
                    {
                        _isFemale = false;
                    }

                    if (_listDic[presetName][number].Contains(":"))
                    {
                        var skinPresetSplit = _listDic[presetName][number].Split(':');
                        _skinPresetFile = skinPresetSplit[0] + ":/" + _presetSystemPath[presetName] + "/" + skinPresetSplit[1] + ".vap";
                    }
                    else
                    {
                        _skinPresetFile = _presetSystemPath[presetName] + "/" + _listDic[presetName][number] + ".vap";
                    }

                    int posi = _skinPresetFile.LastIndexOf("/");
                    _skinPresetFile = _skinPresetFile.Insert(posi + 1, "Preset_");
                    _loadData = SuperController.singleton.ReadFileIntoString(_skinPresetFile);
                    _jSONNode = JSON.Parse(_loadData);
                    _asArray = _jSONNode["storables"].AsArray;

                    for (int i = 0; i < _asArray.Count; i++)
                    {
                        _asObject = _asArray[i].AsObject;
                        _id = _asObject["id"];
                        if (_id == "geometry" && _asObject["character"] != null)
                        {
                            characterName = _asObject["character"];
                        }
                    }

                    bool _isFemaleInVap = _femaleList.Contains(characterName) || characterName.Contains("Female");
                    if (_isFemale == _isFemaleInVap)
                    {
                        _targetAtom.GetStorableByID("SkinPresets").GetStringJSONParam("presetName").val = _listDic[presetName][number];
                        _targetAtom.GetStorableByID("SkinPresets").CallAction("LoadPreset");
                        _current[presetName].val = _listDic[presetName][number];
                    }
                    else
                    {
                        _current[presetName].val = "";
                    }
                }
            }

            presetName = _clothing;
            if (_enablePresetDic[presetName].val)
            {
                if (_listDic[presetName].Count != 0)
                {
                    if (_listDic[presetName].Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_enableRnd.val)
                        {
                            do
                            {
                                number = UnityEngine.Random.Range(0, _listDic[presetName].Count);
                            }
                            while (_previousNumber[presetName] == number);
                            _previousNumber[presetName] = number;
                        }
                        else
                        {
                            number = _previousNumber[presetName];
                            _previousNumber[presetName] = number + 1;
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _previousNumber[presetName] = 0;
                            }
                        }
                    }

                    _targetAtom.GetStorableByID("ClothingPresets").GetStringJSONParam("presetName").val = _listDic[presetName][number];
                    _targetAtom.GetStorableByID("ClothingPresets").CallAction("LoadPreset");
                    _current[presetName].val = _listDic[presetName][number];
                }
            }
            _enablePush = true;
        }

        public void SetClothingPath()
        {
            var presetName = _clothing;
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_presetSystemPath[presetName]);
            _presetPath = _presetSystemPath[presetName];
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetClothingPathAct()
        {
            var presetName = _clothing;
            _presetPath = _presetSystemPath[presetName];
            var path = MakeFilename(_savedPathDic[presetName].val);
            LoadPresetsList(path);
            if (_savedPathDic[presetName].val == "")
            {
                _favPathDic[presetName] = "";
                _favListDic[presetName] = new List<string>();
                _presetPathDic[presetName] = "";
                _presetListDic[presetName] = new List<string>();
                _enablePresetDic[presetName].val = false;
            }
            ChangePath(presetName);
        }

        public void SetHairPath()
        {
            var presetName = _hair;
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_presetSystemPath[presetName]);
            _presetPath = _presetSystemPath[presetName];
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetHairPathAct()
        {
            var presetName = _hair;
            _presetPath = _presetSystemPath[presetName];
            var path = MakeFilename(_savedPathDic[presetName].val);
            LoadPresetsList(path);
            if (_savedPathDic[presetName].val == "")
            {
                _favPathDic[presetName] = "";
                _favListDic[presetName] = new List<string>();
                _presetPathDic[presetName] = "";
                _presetListDic[presetName] = new List<string>();
                _enablePresetDic[presetName].val = false;
            }
            ChangePath(presetName);
        }

        public void SetPosePath()
        {
            var presetName = _pose;
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_presetSystemPath[presetName]);
            _presetPath = _presetSystemPath[presetName];
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetPosePathAct()
        {
            var presetName = _pose;
            _presetPath = _presetSystemPath[presetName];
            var path = MakeFilename(_savedPathDic[presetName].val);
            LoadPresetsList(path);
            if (_savedPathDic[presetName].val == "")
            {
                _favPathDic[presetName] = "";
                _favListDic[presetName] = new List<string>();
                _presetPathDic[presetName] = "";
                _presetListDic[presetName] = new List<string>();
                _enablePresetDic[presetName].val = false;
            }
            ChangePath(presetName);
        }

        public void SetSkinPath()
        {
            var presetName = _skin;
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_presetSystemPath[presetName]);
            _presetPath = _presetSystemPath[presetName];
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetSkinPathAct()
        {
            var presetName = _skin;
            _presetPath = _presetSystemPath[presetName];
            var path = MakeFilename(_savedPathDic[presetName].val);
            LoadPresetsList(path);
            if (_savedPathDic[presetName].val == "")
            {
                _favPathDic[presetName] = "";
                _favListDic[presetName] = new List<string>();
                _presetPathDic[presetName] = "";
                _presetListDic[presetName] = new List<string>();
                _enablePresetDic[presetName].val = false;
            }
            ChangePath(presetName);
        }

        public void SetAppearancePath()
        {
            var presetName = _appearance;
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_presetSystemPath[presetName]);
            _presetPath = _presetSystemPath[presetName];
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetAppearancePathAct()
        {
            var presetName = _appearance;
            _presetPath = _presetSystemPath[presetName];
            var path = MakeFilename(_savedPathDic[presetName].val);
            LoadPresetsList(path);
            if (_savedPathDic[presetName].val == "")
            {
                _favPathDic[presetName] = "";
                _favListDic[presetName] = new List<string>();
                _presetPathDic[presetName] = "";
                _presetListDic[presetName] = new List<string>();
                _enablePresetDic[presetName].val = false;
            }
            ChangePath(presetName);
        }

        IEnumerator MoveRoot()
        {
            if (_min >= 0)
            {
                _min = 0;
            }
            else
            {
                if (_enablePositionAdjust.val)
                {
                    _min -= 0.1f;
                }
                else
                {
                    _min = 0;
                }
            }

            _targetAtom.GetStorableByID("control").transform.position = _personPosi - new Vector3(0, _min, 0);

            yield return new WaitForSeconds(0.5f);
            foreach (var item in _activeClothingList)
            {
                _targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val = true;
            }
            _targetAtom.tempDisableRender = !_targetAtom.tempDisableRender;
            _targetAtom.GetStorableByID("control").GetBoolJSONParam("detachControl").val = true;

            _targetAtom.GetStorableByID("control").transform.position = _personPosi;
            _targetAtom.GetStorableByID("control").transform.rotation = Quaternion.Euler(new Vector3(0, 0, 0));
            yield return new WaitForSeconds(0.5f);
            _targetAtom.GetStorableByID("control").GetBoolJSONParam("detachControl").val = false;
        }

        public string MakeFilename(string path)
        {
            if (path != null && path != "")
            {
                path = path.Replace("\\", "/");
                if (!path.Contains(":"))
                {
                    if (path.Substring(0, 1) == "/")
                    {
                        path = path.Remove(0, 1);
                    }

                    if (!path.Contains(_presetPath))
                    {
                        path = _presetPath + "/" + path;
                    }
                }
                else
                {
                    if (!path.Contains(".var:"))
                    {
                        path = path.Replace(":", ".var:");
                    }

                    string[] div = { ".var:" };
                    var pathSplit = path.Split(div, StringSplitOptions.None);
                    if (!pathSplit[1].Contains(_presetPath))
                    {
                        pathSplit[1] = (_presetPath + "/" + pathSplit[1]).Replace("//", "/");
                    }

                    if (!pathSplit[0].Contains(_addonPackagesPath))
                    {
                        if (FileManagerSecure.FileExists((_addonPackagesPath + "/" + pathSplit[0]).Replace("//", "/")))
                        {
                            pathSplit[0] = (_addonPackagesPath + "/" + pathSplit[0]).Replace("//", "/") + ".var";
                        }
                        else
                        {
                            var filenameSplit = pathSplit[0].Split('.');
                            ArrayList allFiles = new ArrayList();
                            SearchFile(_addonPackagesPath, filenameSplit[0] + "." + filenameSplit[1] + ".*.var", ref allFiles);

                            var searchedFiles = (string[])allFiles.ToArray(typeof(string));
                            int latest = 0;
                            int number = 0;
                            int counter = 0;
                            foreach (var item in searchedFiles)
                            {
                                var itemSplit = item.Replace("\\", "/").Split('/');
                                var findFileSplit = itemSplit[itemSplit.Length - 1].Split('.');
                                if (latest <= int.Parse(findFileSplit[2]))
                                {
                                    latest = int.Parse(findFileSplit[2]);
                                    number = counter;
                                }
                                counter++;
                            }
                            pathSplit[0] = searchedFiles[number];
                        }
                    }
                    else
                    {
                        if (FileManagerSecure.FileExists(pathSplit[0] + ".var"))
                        {
                            pathSplit[0] = pathSplit[0] + ".var";
                        }
                        else
                        {
                            var filePathSplit = pathSplit[0].Split('/');
                            var filenameSplit = filePathSplit[filePathSplit.Length - 1].Split('.');
                            ArrayList allFiles = new ArrayList();
                            SearchFile(_addonPackagesPath, filenameSplit[0] + "." + filenameSplit[1] + ".*.var", ref allFiles);

                            var searchedFiles = (string[])allFiles.ToArray(typeof(string));
                            int latest = 0;
                            int number = 0;
                            int counter = 0;
                            foreach (var item in searchedFiles)
                            {
                                var itemSplit = item.Replace("\\", "/").Split('/');
                                var findFileSplit = itemSplit[itemSplit.Length - 1].Split('.');
                                if (latest <= int.Parse(findFileSplit[2]))
                                {
                                    latest = int.Parse(findFileSplit[2]);
                                    number = counter;
                                }
                                counter++;
                            }
                            pathSplit[0] = searchedFiles[number];
                        }
                    }
                    path = pathSplit[0] + ":/" + pathSplit[1];
                }
                path = path.Replace("//", "/");
                return path;
            }
            return "";
        }

        public void LoadPresetsList(string path)
        {
            if (path != null && path != "")
            {
                path = path.Replace("\\", "/");
                if (!path.Contains(_presetPath) && path.Contains(":"))
                {
                    var pathSplit = path.Split(':');
                }

                _varFileName = "";
                string favPath;
                if (path.Contains(":"))
                {
                    var pathSplit = path.Split(':');
                    _varFullPath = pathSplit[0];
                    var varNameSplit = _varFullPath.Split('/');
                    var varNumber = varNameSplit.Length;
                    _varFullName = varNameSplit[varNumber - 1];
                    _varFileName = _varFullName.Remove(_varFullName.Length - 4, 4);
                    favPath = _filePrefs + _varFileName + pathSplit[1];
                }
                else
                {
                    favPath = path;
                }

                string[] presetFiles;
                string[] favFiles;
                string presetFileList = "";
                string favFileList = "";

                if (_enableSubfolder.val)
                {
                    ArrayList allFiles = new ArrayList();
                    GetAllFiles(path, "Preset_*.vap", ref allFiles);
                    presetFiles = (string[])allFiles.ToArray(typeof(string));
                    _presetList = new List<string>();
                    foreach (var item in presetFiles)
                    {
                        string tempFileName = item.Replace("\\", "/");
                        if (tempFileName.Contains(":"))
                        {
                            tempFileName = _varFileName + ":" + item.Replace("\\", "/").Split(':')[1].Remove(0, 1);
                            tempFileName = tempFileName.Replace(":/", ":");
                        }

                        tempFileName = tempFileName.Replace(_presetPath + "/", "");
                        tempFileName = tempFileName.Remove(tempFileName.LastIndexOf("Preset_"), 7);
                        tempFileName = tempFileName.Remove(tempFileName.LastIndexOf(".vap"), 4);
                        _presetList.Add(tempFileName);
                        if (presetFileList.Length < _jsonstringLimit)
                        {
                            presetFileList = presetFileList + tempFileName + "\n";
                        }
                    }

                    allFiles = new ArrayList();
                    if (FileManagerSecure.DirectoryExists(favPath))
                    {
                        GetAllFiles(favPath, "Preset_*.vap.fav", ref allFiles);
                        favFiles = (string[])allFiles.ToArray(typeof(string));
                        _favList = new List<string>();
                        foreach (var item in favFiles)
                        {
                            string tempFileName = item.Replace("\\", "/");
                            if (tempFileName.Contains(_filePrefs))
                            {
                                var ri = _varFileName.Length;
                                tempFileName = tempFileName.Replace(_filePrefs, "").Remove(ri, 1).Insert(ri, ":");
                            }

                            tempFileName = tempFileName.Replace(_presetPath + "/", "");
                            tempFileName = tempFileName.Remove(tempFileName.LastIndexOf("Preset_"), 7);
                            tempFileName = tempFileName.Remove(tempFileName.LastIndexOf(".vap.fav"), 8);
                            _favList.Add(tempFileName);
                            if (favFileList.Length < _jsonstringLimit)
                            {
                                favFileList = favFileList + tempFileName + "\n";
                            }
                        }
                    }
                    else
                    {
                        _favList = new List<string>();
                        favFileList = "";
                    }
                }
                else
                {
                    presetFiles = FileManagerSecure.GetFiles(path, "Preset_*.vap");
                    _presetList = new List<string>();
                    foreach (var item in presetFiles)
                    {
                        string tempFileName = item.Replace("\\", "/");
                        if (tempFileName.Contains(":"))
                        {
                            tempFileName = _varFileName + ":" + item.Replace("\\", "/").Split(':')[1].Remove(0, 1);
                            tempFileName = tempFileName.Replace(":/", ":");
                        }

                        tempFileName = tempFileName.Replace(_presetPath + "/", "");
                        tempFileName = tempFileName.Remove(tempFileName.LastIndexOf("Preset_"), 7);
                        tempFileName = tempFileName.Remove(tempFileName.LastIndexOf(".vap"), 4);
                        _presetList.Add(tempFileName);
                        if (presetFileList.Length < _jsonstringLimit)
                        {
                            presetFileList = presetFileList + tempFileName + "\n";
                        }
                    }

                    if (FileManagerSecure.DirectoryExists(favPath))
                    {
                        favFiles = FileManagerSecure.GetFiles(favPath, "Preset_*.vap.fav");
                        _favList = new List<string>();
                        foreach (var item in favFiles)
                        {
                            string tempFileName = item.Replace("\\", "/");
                            if (tempFileName.Contains(_filePrefs))
                            {
                                var ri = _varFileName.Length;
                                tempFileName = tempFileName.Replace(_filePrefs, "").Remove(ri, 1).Insert(ri, ":");
                            }

                            tempFileName = tempFileName.Replace(_presetPath + "/", "");
                            tempFileName = tempFileName.Remove(tempFileName.LastIndexOf("Preset_"), 7);
                            tempFileName = tempFileName.Remove(tempFileName.LastIndexOf(".vap.fav"), 8);
                            _favList.Add(tempFileName);
                            if (favFileList.Length < _jsonstringLimit)
                            {
                                favFileList = favFileList + tempFileName + "\n";
                            }
                        }
                    }
                    else
                    {
                        _favList = new List<string>();
                        favFileList = "";
                    }
                }

                if (_presetPath == _presetSystemPath[_clothing])
                {
                    var presetName = _clothing;
                    _presetListDic[presetName] = new List<string>(_presetList);
                    _presetPathDic[presetName] = presetFileList;
                    _favListDic[presetName] = new List<string>(_favList);
                    _favPathDic[presetName] = favFileList;

                    if (_onlyFavDic[presetName].val)
                    {
                        _filesInFolder[presetName].val = _favPathDic[presetName];
                        _listDic[presetName] = new List<string>(_favListDic[presetName]);
                    }
                    else
                    {
                        _filesInFolder[presetName].val = _presetPathDic[presetName];
                        _listDic[presetName] = new List<string>(_presetListDic[presetName]);
                    }
                    _savedPathDic[presetName].val = path;
                    _current[presetName].val = "";
                    _enablePresetDic[presetName].val = true;
                }
                else if (_presetPath == _presetSystemPath[_hair])
                {
                    var presetName = _hair;
                    _presetListDic[presetName] = new List<string>(_presetList);
                    _presetPathDic[presetName] = presetFileList;
                    _favListDic[presetName] = new List<string>(_favList);
                    _favPathDic[presetName] = favFileList;

                    if (_onlyFavDic[presetName].val)
                    {
                        _filesInFolder[presetName].val = _favPathDic[presetName];
                        _listDic[presetName] = new List<string>(_favListDic[presetName]);
                    }
                    else
                    {
                        _filesInFolder[presetName].val = _presetPathDic[presetName];
                        _listDic[presetName] = new List<string>(_presetListDic[presetName]);
                    }
                    _savedPathDic[presetName].val = path;
                    _current[presetName].val = "";
                    _enablePresetDic[presetName].val = true;
                }
                else if (_presetPath == _presetSystemPath[_pose])
                {
                    var presetName = _pose;
                    _presetListDic[presetName] = new List<string>(_presetList);
                    _presetPathDic[presetName] = presetFileList;
                    _favListDic[presetName] = new List<string>(_favList);
                    _favPathDic[presetName] = favFileList;

                    if (_onlyFavDic[presetName].val)
                    {
                        _filesInFolder[presetName].val = _favPathDic[presetName];
                        _listDic[presetName] = new List<string>(_favListDic[presetName]);
                    }
                    else
                    {
                        _filesInFolder[presetName].val = _presetPathDic[presetName];
                        _listDic[presetName] = new List<string>(_presetListDic[presetName]);
                    }
                    _savedPathDic[presetName].val = path;
                    _current[presetName].val = "";
                    _enablePresetDic[presetName].val = true;
                }
                else if (_presetPath == _presetSystemPath[_skin])
                {
                    var presetName = _skin;
                    _presetListDic[presetName] = new List<string>(_presetList);
                    _presetPathDic[presetName] = presetFileList;
                    _favListDic[presetName] = new List<string>(_favList);
                    _favPathDic[presetName] = favFileList;

                    if (_onlyFavDic[presetName].val)
                    {
                        _filesInFolder[presetName].val = _favPathDic[presetName];
                        _listDic[presetName] = new List<string>(_favListDic[presetName]);
                    }
                    else
                    {
                        _filesInFolder[presetName].val = _presetPathDic[presetName];
                        _listDic[presetName] = new List<string>(_presetListDic[presetName]);
                    }
                    _savedPathDic[presetName].val = path;
                    _current[presetName].val = "";
                    _enablePresetDic[presetName].val = true;
                }
                else
                {
                    var presetName = _appearance;
                    _presetListDic[presetName] = new List<string>(_presetList);
                    _presetPathDic[presetName] = presetFileList;
                    _favListDic[presetName] = new List<string>(_favList);
                    _favPathDic[presetName] = favFileList;

                    if (_onlyFavDic[presetName].val)
                    {
                        _filesInFolder[presetName].val = _favPathDic[presetName];
                        _listDic[presetName] = new List<string>(_favListDic[presetName]);
                    }
                    else
                    {
                        _filesInFolder[presetName].val = _presetPathDic[presetName];
                        _listDic[presetName] = new List<string>(_presetListDic[presetName]);
                    }
                    _savedPathDic[presetName].val = path;
                    _current[presetName].val = "";
                    _enablePresetDic[presetName].val = true;
                }
            }
        }

        public static void GetAllFiles(string folder, string filter, ref ArrayList allFiles)
        {
            string[] filses = FileManagerSecure.GetFiles(folder, filter);
            allFiles.AddRange(filses);

            string[] subFolders = FileManagerSecure.GetDirectories(folder);
            foreach (string subF in subFolders)
            {
                GetAllFiles(subF, filter, ref allFiles);
            }
        }

        public static void SearchFile(string folder, string filter, ref ArrayList allFiles)
        {
            string[] filses = FileManagerSecure.GetFiles(folder, filter);
            allFiles.AddRange(filses);

            string[] subFolders = FileManagerSecure.GetDirectories(folder);
            foreach (string subF in subFolders)
            {
                SearchFile(subF, filter, ref allFiles);
            }
        }

        public void RefreshList()
        {
            var presetName = _clothing;
            if (_savedPathDic[presetName] != null && _savedPathDic[presetName].val != "")
            {
                _presetPath = _presetSystemPath[presetName];
                _listDic[presetName] = new List<string>();
                var path = MakeFilename(_savedPathDic[presetName].val);
                LoadPresetsList(path);
                ChangePath(presetName);
            }

            presetName = _hair;
            if (_savedPathDic[presetName] != null && _savedPathDic[presetName].val != "")
            {
                _presetPath = _presetSystemPath[presetName];
                _listDic[presetName] = new List<string>();
                var path = MakeFilename(_savedPathDic[presetName].val);
                LoadPresetsList(path);
                ChangePath(presetName);
            }

            presetName = _pose;
            if (_savedPathDic[presetName] != null && _savedPathDic[presetName].val != "")
            {
                _presetPath = _presetSystemPath[presetName];
                _listDic[presetName] = new List<string>();
                var path = MakeFilename(_savedPathDic[presetName].val);
                LoadPresetsList(path);
                ChangePath(presetName);
            }

            presetName = _skin;
            if (_savedPathDic[presetName] != null && _savedPathDic[presetName].val != "")
            {
                _presetPath = _presetSystemPath[presetName];
                _listDic[presetName] = new List<string>();
                var path = MakeFilename(_savedPathDic[presetName].val);
                LoadPresetsList(path);
                ChangePath(presetName);
            }

            presetName = _appearance;
            if (_savedPathDic[presetName] != null && _savedPathDic[presetName].val != "")
            {
                _presetPath = _presetSystemPath[presetName];
                _listDic[presetName] = new List<string>();
                var path = MakeFilename(_savedPathDic[presetName].val);
                LoadPresetsList(path);
                ChangePath(presetName);
            }
        }

        // Start is called once before Update or FixedUpdate is called and after Init()
        void Start()
        {
            try
            {
                var presetName = _clothing;
                if (_savedPathDic[presetName] != null && _savedPathDic[presetName].val != "")
                {
                    _presetPath = _presetSystemPath[presetName];
                    _listDic[presetName] = new List<string>();
                    var path = MakeFilename(_savedPathDic[presetName].val);
                    LoadPresetsList(path);
                    ChangePath(presetName);
                }

                presetName = _hair;
                if (_savedPathDic[presetName] != null && _savedPathDic[presetName].val != "")
                {
                    _presetPath = _presetSystemPath[presetName];
                    _listDic[presetName] = new List<string>();
                    var path = MakeFilename(_savedPathDic[presetName].val);
                    LoadPresetsList(path);
                    ChangePath(presetName);
                }

                presetName = _pose;
                if (_savedPathDic[presetName] != null && _savedPathDic[presetName].val != "")
                {
                    _presetPath = _presetSystemPath[presetName];
                    _listDic[presetName] = new List<string>();
                    var path = MakeFilename(_savedPathDic[presetName].val);
                    LoadPresetsList(path);
                    ChangePath(presetName);
                }

                presetName = _skin;
                if (_savedPathDic[presetName] != null && _savedPathDic[presetName].val != "")
                {
                    _presetPath = _presetSystemPath[presetName];
                    _listDic[presetName] = new List<string>();
                    var path = MakeFilename(_savedPathDic[presetName].val);
                    LoadPresetsList(path);
                    ChangePath(presetName);
                }

                presetName = _appearance;
                if (_savedPathDic[presetName] != null && _savedPathDic[presetName].val != "")
                {
                    _presetPath = _presetSystemPath[presetName];
                    _listDic[presetName] = new List<string>();
                    var path = MakeFilename(_savedPathDic[presetName].val);
                    LoadPresetsList(path);
                    ChangePath(presetName);
                }
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            try
            {

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // FixedUpdate is called with each physics simulation frame by Unity
        void FixedUpdate()
        {
            try
            {
                if (_previousSelectCount != _selectCount.val)
                {
                    _waitCount = 0;
                }

                if (!_enablePresetDic[_clothing].val)
                {
                    _previousNumber[_clothing] = 0;
                }

                if (!_enablePresetDic[_hair].val)
                {
                    _previousNumber[_hair] = 0;
                }

                if (!_enablePresetDic[_pose].val)
                {
                    _previousNumber[_pose] = 0;
                }

                if (!_enablePresetDic[_skin].val)
                {
                    _previousNumber[_skin] = 0;
                }

                if (!_enablePresetDic[_appearance].val)
                {
                    _previousNumber[_appearance] = 0;
                }

                _selectTime = int.Parse(_selectCount.val);
                if (_enableAutoLoad.val && _loopStart)
                {
                    if (_waitCount <= 0)
                    {
                        _waitCount = _selectTime;
                        _startTime = DateTime.Now.DayOfYear * 24 * 60 * 60 + DateTime.Now.Hour * 60 * 60 + DateTime.Now.Minute * 60 + DateTime.Now.Second;
                        _enablePush = true;
                        LoadPreset();
                        _currentTime = DateTime.Now.DayOfYear * 24 * 60 * 60 + DateTime.Now.Hour * 60 * 60 + DateTime.Now.Minute * 60 + DateTime.Now.Second;
                        if (_selectTime < _currentTime - _startTime)
                        {
                            SuperController.LogMessage("Please select interval time more than " + (_currentTime - _startTime + 1).ToString());
                            _selectCount.val = "30";
                            _selectTime = int.Parse(_selectCount.val);
                        }
                    }
                    _currentTime = DateTime.Now.DayOfYear * 24 * 60 * 60 + DateTime.Now.Hour * 60 * 60 + DateTime.Now.Minute * 60 + DateTime.Now.Second;
                    _waitCount = _selectTime - (_currentTime - _startTime);
                }
                else
                {
                    _loopStart = false;
                    _waitCount = _selectTime;
                }

                _previousSelectCount = _selectCount.val;
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // OnDestroy is where you should put any cleanup
        // if you registered objects to supercontroller or atom, you should unregister them here
        void OnDestroy()
        {

        }
    }
}